@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1997 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dproc.c,v 1.22 2008/10/21 16:17:21 abe Exp $";
+static char *rcsid = "$Id: dproc.c,v 1.23 2010/07/29 16:04:52 abe Exp $";
 #endif
 
 #include "lsof.h"
@@ -87,7 +87,7 @@ _PROTOTYPE(static int read_id_stat,(int ty, char *p, int id, char **cmd,
 				    int *ppid, int *pgid));
 _PROTOTYPE(static void process_proc_map,(char *p, struct stat *s, int ss));
 _PROTOTYPE(static int process_id,(char *idp, int idpl, char *cmd, UID_ARG uid,
-				  int pid, int ppid, int pgid));
+				  int pid, int ppid, int pgid, int tid));
 _PROTOTYPE(static int statEx,(char *p, struct stat *s, int *ss));
  
 
@@ -155,20 +155,21 @@ enter_cntx_arg(cntx)
 void
 gather_proc_info()
 {
-	char *cmd;
+	char *cmd, *tcmd;
 	struct dirent *dp;
-	struct stat sb;
-	int lwp, n, nl, pgid, pid, ppid, rv, tx;
-	static char *lwppath = (char *)NULL;
-	static int lwppathl = 0;
+	unsigned char ht, pidts;
+	int n, nl, pgid, pid, ppid, rv, tid, tpgid, tppid, tx;
 	static char *path = (char *)NULL;
 	static int pathl = 0;
 	static char *pidpath = (char *)NULL;
 	static MALLOC_S pidpathl = 0;
 	static MALLOC_S pidx = 0;
 	static DIR *ps = (DIR *)NULL;
+	struct stat sb;
 	static char *taskpath = (char *)NULL;
 	static int taskpathl = 0;
+	static char *tidpath = (char *)NULL;
+	static int tidpathl = 0;
 	DIR *ts;
 	UID_ARG uid;
 
@@ -273,72 +274,94 @@ gather_proc_info()
 	    if (stat(pidpath, &sb))
 		continue;
 	    uid = (UID_ARG)sb.st_uid;
-	/*
-	 * Process the PID's process information.
-	 */
-	    (void) make_proc_path(pidpath, n, &path, &pathl, "stat");
-	    rv = read_id_stat(0, path, pid, &cmd, &ppid, &pgid);
-	    if (rv == 1)
-		continue;
-	    else if (rv == 0) {
-		(void) process_id(pidpath, n, cmd, uid, pid, ppid, pgid);
-		continue;
-	    }
-	/*
-	 * The process is a zombie.  Check for a non-zombie task.
-	 */
-	    (void) make_proc_path(pidpath, n, &taskpath, &taskpathl, "task");
-	    tx = n + 4;
-	    if ((ts = opendir(taskpath))) {
+	    ht = pidts = 0;
 
-	    /*
-	     * Process the PID's tasks (lightweight processes.)  Record the
-	     * open files of the first one whose LWP ID does not match the PID
-	     * and which is not a itself a zombie.
-	     */
-		while ((dp = readdir(ts))) {
+#if	defined(HASTASKS)
+	/*
+	 * If task reporting is selected, check the tasks of the process first,
+	 * so that the "-p<PID> -aK" options work properly.
+	 */
+	    if ((Selflags & SELTASK)) {
+		(void) make_proc_path(pidpath, n, &taskpath, &taskpathl,
+				      "task");
+		tx = n + 4;
+		if ((ts = opendir(taskpath))) {
 
 		/*
-		 * Get the LWP ID.  Skip the task if its LWP ID matches the
-		 * process PID.
+		 * Process the PID's tasks.  Record the open files of those
+		 * whose TIDs do not match the PID and which are themselves
+		 * not zombies.
 		 */
-		    if (nm2id(dp->d_name, &lwp, &nl))
-			continue;
-		    if  (lwp == pid)
-			continue;
-		/*
-		 * Check the LWP state.
-		 */
-		    if (read_id_stat(1, path, lwp, &cmd, &ppid, &pgid))
-			continue;
-		/*
-		 * Attempt to record the LWP.
-		 */
-		    if ((tx + 1 + nl + 1) > lwppathl) {
-			lwppathl = tx + 1 + n + 1 + 64;
-			if (lwppath)
-			    lwppath = (char *)realloc((MALLOC_P *)lwppath,
-						      lwppathl);
-			else
-			    lwppath = (char *)malloc((MALLOC_S)lwppathl);
-			if (!lwppath) {
-			    (void) fprintf(stderr,
-				"%s: can't allocate %d task bytes", Pn,
-				lwppathl);
-			    (void) fprintf(stderr,
-				" for \"%s/%s/\"\n", taskpath, dp->d_name);
-			    Exit(1);
+		    while ((dp = readdir(ts))) {
+
+		    /*
+		     * Get the task ID.  Skip the task if its ID matches the
+		     * process PID.
+		     */
+			if (nm2id(dp->d_name, &tid, &nl))
+			    continue;
+			if  (tid == pid) {
+			    pidts = 1;
+			    continue;
+			}
+		    /*
+		     * Form the path for the TID.
+		     */
+			if ((tx + 1 + nl + 1 + 4) > tidpathl) {
+			    tidpathl = tx + 1 + n + 1 + 4 + 64;
+			    if (tidpath)
+				tidpath = (char *)realloc((MALLOC_P *)tidpath,
+							  tidpathl);
+			    else
+				tidpath = (char *)malloc((MALLOC_S)tidpathl);
+			    if (!tidpath) {
+				(void) fprintf(stderr,
+				    "%s: can't allocate %d task bytes", Pn,
+				    tidpathl);
+				(void) fprintf(stderr, " for \"%s/%s/stat\"\n",
+				    taskpath, dp->d_name);
+				Exit(1);
+			    }
+			}
+			(void) snpf(tidpath, tidpathl, "%s/%s/stat", taskpath,
+			    dp->d_name);
+		    /*
+		     * Check the task state.
+		     */
+			rv = read_id_stat(1, tidpath, tid, &tcmd, &tppid,
+					  &tpgid);
+			if ((rv < 0) || (rv == 1))
+			    continue;
+		    /*
+		     * Attempt to record the task.
+		     */
+			if (!process_id(tidpath, (tx + 1 + nl+ 1), tcmd, uid,
+					pid, tppid, tpgid, tid))
+			{
+			    ht = 1;
 			}
 		    }
-		    (void) snpf(lwppath, lwppathl, "%s/%s/", taskpath,
-			dp->d_name);
-		    if (!process_id(lwppath, (tx + 1 + nl+ 1), cmd, uid, pid,
-				    ppid, pgid))
-		    {
-			break;
-		    }
+		    (void) closedir(ts);
+		}
+	    }
+#endif	/* defined(HASTASKS) */
+
+	/*
+	 * If the main process is a task and task selection has been specified
+	 * along with option ANDing, enter the main process temporarily as a
+	 * task,  so that the "-aK" option set lists the main process along
+	 * with its tasks.
+	 */
+	    (void) make_proc_path(pidpath, n, &path, &pathl, "stat");
+	    if (((rv = read_id_stat(0, path, pid, &cmd, &ppid, &pgid)) >= 0) 
+	    &&   (rv != 1))
+	    {
+		tid = (Fand && ht && pidts && (Selflags & SELTASK)) ? pid : 0;
+		if ((!process_id(pidpath, n, cmd, uid, pid, ppid, pgid, tid))
+		&&  tid)
+		{
+		    Lp->tid = 0;
 		}
-		(void) closedir(ts);
 	    }
 	}
 }
@@ -665,7 +688,7 @@ open_proc_stream(p, m, buf, sz, act)
  */
 
 static int
-process_id(idp, idpl, cmd, uid, pid, ppid, pgid)
+process_id(idp, idpl, cmd, uid, pid, ppid, pgid, tid)
 	char *idp;			/* pointer to ID's path */
 	int idpl;			/* pointer to ID's path length */
 	char *cmd;			/* pointer to ID's command */
@@ -673,11 +696,12 @@ process_id(idp, idpl, cmd, uid, pid, ppid, pgid)
 	int pid;			/* ID's PID */
 	int ppid;			/* parent PID */
 	int pgid;			/* parent GID */
+	int tid;			/* task ID, if non-zero */
 {
 	int av;
 	static char *dpath = (char *)NULL;
 	static int dpathl = 0;
-	short enls, enss, lnk, oty, pn, pss, sf;
+	short enls, enss, lnk, oty, pn, pss, sf, tsf;
 	int fd, i, ls, n, ss, sv;
 	struct l_fdinfo fi;
 	DIR *fdp;
@@ -700,7 +724,7 @@ process_id(idp, idpl, cmd, uid, pid, ppid, pgid)
 /*
  * See if process is excluded.
  */
-	if (is_proc_excl(pid, pgid, uid, &pss, &sf)
+	if (is_proc_excl(pid, pgid, uid, &pss, &sf, tid)
 	||  is_cmd_excl(cmd, &pss, &sf))
 	    return(1);
 	if (Cckreg) {
@@ -713,6 +737,7 @@ process_id(idp, idpl, cmd, uid, pid, ppid, pgid)
 	    Ckscko = (sf & SELPROC) ? 0 : 1;
 	}
 	alloc_lproc(pid, pgid, ppid, uid, cmd, (int)pss, (int)sf);
+	Lp->tid = tid;
 	Plf = (struct lfile *)NULL;
 /*
  * Process the ID's current working directory info.
@@ -1207,9 +1232,10 @@ process_proc_map(p, s, ss)
 /*
  * read_id_stat() - read ID (PID or LWP ID) status
  *
- * return: -1 == ID is a zombie
+ * return: -1 == ID is unavailable
  *          0 == ID OK
- *          1 == ID unavailable
+ *          1 == ID is a zombie
+ *	    2 == ID is a thread
  */
 
 static int
@@ -1235,11 +1261,11 @@ read_id_stat(ty, p, id, cmd, ppid, pgid)
  * and read the file's first line.
  */
 	if (!(fs = open_proc_stream(p, "r", &vbuf, &vsz, 0)))
-	    return(1);
+	    return(-1);
 	cp = fgets(buf, sizeof(buf), fs);
 	(void) fclose(fs);
 	if (!cp)
-	    return(1);
+	    return(-1);
 /*
  * Separate the line into fields on white space separators.  Expect five fields
  * for a PID type and three for an LWP type.
@@ -1247,21 +1273,21 @@ read_id_stat(ty, p, id, cmd, ppid, pgid)
 	if ((nf = get_fields(buf, (char *)NULL, &fp, (int *)NULL, 0))
 	<  (ty ? 5 : 3))
 	{
-	    return(1);
+	    return(-1);
 	}
 /*
  * Convert the first field to an integer; its conversion must match the
  * ID argument.
  */
 	if (!fp[0] || (atoi(fp[0]) != id))
-	    return(1);
+	    return(-1);
 /*
  * Get the command name from the second field.  Strip a starting '(' and
  * an ending ')'.  Allocate space to hold the result and return the space
  * pointer.
  */
 	if (!(cp = fp[1]))
-	    return(1);
+	    return(-1);
 	if (cp && *cp == '(')
 	    cp++;
 	if ((cp1 = strrchr(cp, ')')))
@@ -1282,23 +1308,25 @@ read_id_stat(ty, p, id, cmd, ppid, pgid)
 	(void) snpf(cbf, len, "%s", cp);
 	*cmd = cbf;
 /*
- * If the type is PID, convert and return parent process (fourth field)
- * and process group (fifth field) IDs.
+ * Convert and return parent process (fourth field) and process group (fifth
+ * field) IDs.
  */
-	if (!ty) {
-	    if (fp[3] && *fp[3])
-		*ppid = atoi(fp[3]);
-	    else
-		return(1);
-	    if (fp[4] && *fp[4])
-		*pgid = atoi(fp[4]);
-	    else
-		return(1);
-	}
+	if (fp[3] && *fp[3])
+	    *ppid = atoi(fp[3]);
+	else
+	    return(-1);
+	if (fp[4] && *fp[4])
+	    *pgid = atoi(fp[4]);
+	else
+	    return(-1);
 /*
  * Check the state in the third field.  If it is 'Z', return that indication.
  */
-	return((fp[2] && !strcmp(fp[2], "Z")) ? -1 : 0);
+	if (fp[2] && !strcmp(fp[2], "Z"))
+	    return(1);
+	else if (fp[2] && !strcmp(fp[2], "T"))
+	    return(2);
+	return(0);
 }
 
 
