@@ -31,7 +31,7 @@
 
 
 /*
- * $Id: machine.h,v 1.33 2008/10/21 16:17:21 abe Exp $
+ * $Id: machine.h,v 1.34 2010/07/29 16:02:50 abe Exp $
  */
 
 
@@ -488,6 +488,13 @@
 /* #define	HASSTREAMS	1 */
 
 
+/*
+ * HASTASKS is defined for those dialects that have task reporting support.
+ */
+
+#define	HASTASKS	1
+
+
 /*
  * HASTCPTPIQ is defined for dialects where it is possible to report the
  * TCP/TPI Recv-Q and Send-Q values produced by netstat.
