@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dsock.c,v 1.29 2010/01/18 19:03:54 abe Exp $";
+static char *rcsid = "$Id: dsock.c,v 1.30 2010/07/29 16:05:15 abe Exp $";
 #endif
 
 
@@ -62,16 +62,28 @@ static char *rcsid = "$Id: dsock.c,v 1.29 2010/01/18 19:03:54 abe Exp $";
 
 typedef struct icmp_s {
 	uint_t icmp_state;		/* TPI state */
+	KA_T *icmp_connp;		/* connection structure pointer */
 	in6_addr_t icmp_bound_v6src;	/* Explicitely bound to address */
 	in6_addr_t icmp_v6src;		/* Source address of this stream */
-	uint_t icmp_debug;		/* This name identifies a single bit
-					 * variable, but CTF won't read
-					 * individual bit variables, so for
-					 * CTF's purposes it is declared as a
-					 * single member, named by its first
-					 * bit variable, whose address CTF
-					 * groks.  The bit variables are later
-					 * decoded via a union. */
+	union {
+	    uint_t icmp_dummy;
+	    uint_t
+		icmp_Debug : 1,	   	/* SO_DEBUG option */
+		icmp_dontroute : 1,	/* SO_DONTROUTE option */
+		icmp_broadcast : 1,	/* SO_BROADCAST option */
+		icmp_reuseaddr : 1,	/* SO_REUSEADDR option */
+		icmp_useloopback : 1,	/* SO_USELOOPBACK option */
+		icmp_hdrincl : 1,	/* IP_HDRINCL option, etc. */
+		icmp_dgram_errind : 1,	/* SO_DGRAM_ERRIND option */
+		icmp_pad : 25;		/* pad to bit 31 */
+	} icmp_debug;			/* This name identifies a single bit
+					 * variable of the kernel's union, but
+					 * CTF won't read individual bit
+					 * variables, so for CTF's purposes
+					 * it is declared as a uint_t union,
+					 * named by the first bit variable of
+					 * the kernel union, whose address CTF
+					 * groks. */
 } icmp_t;
 
 
@@ -81,28 +93,40 @@ typedef struct icmp_s {
 
 typedef struct rts_s {
 	uint_t rts_state;		/* Provider interface state */
-	uint_t rts_debug;		/* This name identifies a single bit
+
+# if	defined(HAS_CONN_NEW)
+	KA_T *rts_connp;		/* connection structure pointer */
+# endif	/* defined(HAS_CONN_NEW) */
+
+	union {
+	    uint_t rts_dummy;
+	    uint_t
+		rts_Debug : 1,		/* SO_DEBUG option */
+		rts_dontroute : 1,	/* SO_DONTROUTE option */
+		rts_broadcast : 1,	/* SO_BROADCAST option */
+		rts_reuseaddr : 1,	/* SO_REUSEADDR option */
+		rts_useloopback : 1,	/* SO_USELOOPBACK option */
+		icmp_pad : 27;		/* padding to bit 31 */
+	} rts_debug;			/* This name identifies a single bit
 					 * variable, but CTF won't read
 					 * individual bit variables, so for
 					 * CTF's purposes it is declared as a
-					 * single member, named by its first
+					 * uint_t union, named by its first
 					 * bit variable, whose address CTF
-					 * groks.  The bit variables are later
-					 * decoded via a union. */
+					 * groks. */
 } rts_t;
 
 /*
  * Udp_t internal structure definition
  */
 
-typedef struct udp {
-	uint_t udp_state;		/* TPI state */
+typedef struct udp { uint_t udp_state;		/* TPI state */
 	in_port_t udp_port;		/* port bound to this stream */
 	in_port_t udp_dstport;		/* connected port */
 	in6_addr_t udp_v6src;		/* source address of this stream */
 	in6_addr_t udp_v6dst;		/* connected destination */
 	ushort_t udp_ipversion;		/* version -- IPV[46]_VERSION */
-	KA_T udp_connp;			/* connection structure pointer */
+	KA_T *udp_connp; 		/* connection structure pointer */
 	uint_t udp_bits;		/* socket option bits */
 } udp_t;
 
@@ -130,12 +154,21 @@ static	CTF_member_t icmp_t_members[] = {
     CTF_MEMBER(icmp_state),
 #define	MX_icmp_state			0
 
+# if	defined(HAS_CONN_NEW)
+
+    CTF_MEMBER(icmp_connp),
+#define	MX_icmp_connp			1
+# else	/* !defined(HAS_CONN_NEW) */
     CTF_MEMBER(icmp_bound_v6src),
 #define	MX_icmp_bound_v6src		1
 
     CTF_MEMBER(icmp_v6src),
 #define	MX_icmp_v6src			2
 
+    CTF_MEMBER(icmp_debug),
+#define	MX_icmp_debug			3
+# endif	/* defined(HAS_CONN_NEW) */
+
     { NULL, 0 }
 };
 
@@ -146,8 +179,14 @@ static	CTF_member_t rts_t_members[] = {
     CTF_MEMBER(rts_state),
 #define	MX_rts_state			0
 
+# if	defined(HAS_CONN_NEW)
+    CTF_MEMBER(rts_connp),
+#define	MX_rts_connp			1
+# else	/* !defined(HAS_CONN_NEW) */
+
     CTF_MEMBER(rts_debug),
 #define	MX_rts_debug			1
+# endif	/* defined(HAS_CONN_NEW) */
 
     { NULL, 0 }
 };
@@ -159,26 +198,28 @@ static	CTF_member_t udp_t_members[] = {
     CTF_MEMBER(udp_state),
 #define	MX_udp_state			0
 
+    CTF_MEMBER(udp_connp),
+#define	MX_udp_connp			1
+
+# if	!defined(HAS_CONN_NEW)
     CTF_MEMBER(udp_port),
-#define	MX_udp_port			1
+#define	MX_udp_port			2
 
     CTF_MEMBER(udp_dstport),
-#define	MX_udp_dstport			2
+#define	MX_udp_dstport			3
 
     CTF_MEMBER(udp_v6src),
-#define	MX_udp_v6src			3
+#define	MX_udp_v6src			4
 
     CTF_MEMBER(udp_v6dst),
-#define	MX_udp_v6dst			4
+#define	MX_udp_v6dst			5
 
     CTF_MEMBER(udp_ipversion),
-#define	MX_udp_ipversion		5
-
-    CTF_MEMBER(udp_connp),
-#define	MX_udp_connp			6
+#define	MX_udp_ipversion		6
 
     CTF_MEMBER(udp_bits),
 #define	MX_udp_bits			7
+# endif	/* !defined(HAS_CONN_NEW) */
 
     { NULL, 0 }
 };
@@ -274,7 +315,8 @@ typedef struct tcpb {
  */
 
 _PROTOTYPE(static void save_TCP_size,(tcp_t *tc));
-_PROTOTYPE(static void save_TCP_states,(tcp_t *tc, tcph_t *th, tcpb_t *tb));
+_PROTOTYPE(static void save_TCP_states,(tcp_t *tc, caddr_t *fa, tcpb_t *tb,
+					caddr_t *xp));
 
 
 /*
@@ -647,6 +689,17 @@ print_tcptpi(nl)
  * procss_VSOCK() -- process a VSOCK socket
  */
 
+# if	defined(HAS_CONN_NEW)
+/*
+ * Adjust for changes in the conn_s structure, introduced at OpenSolaris
+ * level b134.
+ */
+
+#define	conn_ulp conn_proto
+#define conn_rem V4_PART_OF_V6(connua_v6addr.connua_faddr)
+#define conn_src V4_PART_OF_V6(connua_v6addr.connua_laddr)
+# endif	/* defined(HAS_CONN_NEW) */
+
 int
 process_VSOCK(va, v, so)
 	KA_T va;			/* containing vnode address */
@@ -671,7 +724,14 @@ process_VSOCK(va, v, so)
 	char tbuf[32], tbuf1[32];	/* temporary buffers */
 	tcp_t tc;			/* TCP control structure */
 	tcph_t th;			/* TCP header structure */
+
+# if	defined(HAS_CONN_NEW)
+	struct ip_xmit_attr_s xa;
+	caddr_t *xp = (caddr_t *)NULL;
+# else	/* !defined(HAS_CONN_NEW) */
 	tcph_t *tha = (tcph_t *)NULL;	/* TCP header structure address */
+# endif	/* defined(HAS_CONN_NEW) */
+
 	char *ty;			/* TCP type */
 	udp_t uc;			/* local UDP control structure */
 /*
@@ -796,13 +856,25 @@ process_VSOCK(va, v, so)
 	    /*
 	     * Save TCP state information.
 	     */
+
+# if	defined(HAS_CONN_NEW)
+		if ((ka = (KA_T)cs.conn_ixa)
+		&&  !kread(ka, (char *)&xa, sizeof(xa))
+		) {
+		    xp = (caddr_t *)&xa;
+		}
+		(void) save_TCP_states(&tc, (caddr_t *)&cs, (tcpb_t *)NULL, xp);
+# else	/* !defined(HAS_CONN_NEW) */
 		if (tc.tcp_tcp_hdr_len
 		&&  (ka = (KA_T)tc.tcp_tcph)
 		&&  !kread(ka, (char *)&th, sizeof(th))
 		) {
 		    tha = &th;
 		}
-		(void) save_TCP_states(&tc, tha, (tcpb_t *)NULL);
+		(void) save_TCP_states(&tc, (caddr_t *)tha, (tcpb_t *)NULL,
+				       (caddr_t *)NULL);
+# endif	/* defined(HAS_CONN_NEW) */
+
 		Lf->lts.type = 0;
 		Lf->lts.state.i = (int)tc.tcp_state;
 	    /*
@@ -888,7 +960,7 @@ process_VSOCK(va, v, so)
 		Lf->lts.type = 1;
 		Lf->lts.state.ui = (unsigned int)uc.udp_state;
 
-#if	defined(HASSOOPT)
+# if	defined(HASSOOPT)
 	    /*
 	     * Save UDP flags.
 	     */
@@ -919,7 +991,7 @@ process_VSOCK(va, v, so)
 		    if (ucf.udpb_dgram_errind)
 			Lf->lts.opt |= SO_DGRAM_ERRIND;
 		}
-#endif	/* defined(HASSOOPT) */
+# endif	/* defined(HASSOOPT) */
 
 		break;
 	    case IPPROTO_ICMP:
@@ -974,39 +1046,25 @@ process_VSOCK(va, v, so)
 		if (la || fa)
 		    (void)ent_inaddr(la, 0, fa, 0, af);
 
-#if	defined(HASSOOPT)
+# if	defined(HASSOOPT)
 	    /*
 	     * Save ICMP flags.
 	     */
 		if (Ftcptpi & TCPTPI_FLAGS) {
-		    union {
-			uint_t flags;
-			uint_t
-			    icmp_debug : 1,	   /* SO_DEBUG option */
-			    icmp_dontroute : 1,	   /* SO_DONTROUTE option */
-			    icmp_broadcast : 1,	   /* SO_BROADCAST option */
-			    icmp_reuseaddr : 1,	   /* SO_REUSEADDR option */
-			    icmp_useloopback : 1,  /* SO_USELOOPBACK option */
-			    icmp_hdrincl : 1,	   /* IP_HDRINCL option, etc. */
-			    icmp_dgram_errind : 1, /* SO_DGRAM_ERRIND option */
-			    icmp_pad : 25;	   /* pad to bit 31 */
-		    } icf;
-
-		    icf.flags = ic.icmp_debug;
-		    if (icf.icmp_debug)
+		    if (ic.icmp_debug.icmp_Debug)
 			Lf->lts.opt |= SO_DEBUG;
-		    if (icf.icmp_dontroute)
+		    if (ic.icmp_debug.icmp_dontroute)
 		        Lf->lts.opt |= SO_DONTROUTE;
-		    if (icf.icmp_broadcast)
+		    if (ic.icmp_debug.icmp_broadcast)
 		        Lf->lts.opt |= SO_BROADCAST;
-		    if (icf.icmp_reuseaddr)
+		    if (ic.icmp_debug.icmp_reuseaddr)
 			Lf->lts.opt |= SO_REUSEADDR;
-		    if (icf.icmp_useloopback)
+		    if (ic.icmp_debug.icmp_useloopback)
 			Lf->lts.opt |= SO_USELOOPBACK;
-		    if (icf.icmp_dgram_errind)
+		    if (ic.icmp_debug.icmp_dgram_errind)
 			Lf->lts.opt |= SO_DGRAM_ERRIND;
 		}
-#endif	/* defined(HASSOOPT) */
+# endif	/* defined(HASSOOPT) */
 
 		break;
 	    default:
@@ -1061,35 +1119,23 @@ process_VSOCK(va, v, so)
 		}
 	    }
 
-#if	defined(HASSOOPT)
+# if	defined(HASSOOPT)
 	/*
 	 * Save ROUTE flags.
 	 */
 	    if (Ftcptpi & TCPTPI_FLAGS) {
-		union {
-		    uint_t flags;
-		    uint_t
-			rts_debug : 1,		/* SO_DEBUG option */
-			rts_dontroute : 1,	/* SO_DONTROUTE option */
-			rts_broadcast : 1,	/* SO_BROADCAST option */
-			rts_reuseaddr : 1,	/* SO_REUSEADDR option */
-			rts_useloopback : 1,	/* SO_USELOOPBACK option */
-			icmp_pad : 27;		/* padding to bit 31 */
-		} rtf;
-
-		rtf.flags = rt.rts_debug;
-		if (rtf.rts_debug)
+		if (rt.rts_debug.rts_Debug)
 		    Lf->lts.opt |= SO_DEBUG;
-		if (rtf.rts_dontroute)
+		if (rt.rts_debug.rts_dontroute)
 		    Lf->lts.opt |= SO_DONTROUTE;
-		if (rtf.rts_broadcast)
+		if (rt.rts_debug.rts_broadcast)
 		    Lf->lts.opt |= SO_BROADCAST;
-		if (rtf.rts_reuseaddr)
+		if (rt.rts_debug.rts_reuseaddr)
 		   Lf->lts.opt |= SO_REUSEADDR;
-		if (rtf.rts_useloopback)
+		if (rt.rts_debug.rts_useloopback)
 		    Lf->lts.opt |= SO_USELOOPBACK;
 	    }
-#endif	/* defined(HASSOOPT) */
+# endif	/* defined(HASSOOPT) */
 
 	    break;
 	default:
@@ -1671,7 +1717,8 @@ process_socket(sa, ty)
 	     * Save TCP state information.
 	     */
 		if (tcs) {
-		    (void) save_TCP_states(&tc, tha, tcbp);
+		    (void) save_TCP_states(&tc, (caddr_t *)tha, tcbp,
+					   (caddr_t *)NULL);
 		    Lf->lts.type = 0;
 		    Lf->lts.state.i = (int)tc.tcp_state;
 		}
@@ -1707,11 +1754,25 @@ read_icmp_t(va, ph, ia, ic)
 {
 	char tbuf[32], tbuf1[32];	/* print_kptr() temporary buffers */
 
+# if	defined(HAS_CONN_NEW)
+	struct conn_s cs;		/* connection structure */
+	KA_T ka;			/* kernel address */
+
+	zeromem((char *)ic, sizeof(icmp_t));
+# endif	/* defined(HAS_CONN_NEW) */
+
 	(void) CTF_init(&IRU_ctfs, IRU_MOD_FORMAT, IRU_requests);
 	if (!ia
 	||  CTF_MEMBER_READ(ia, ic, icmp_t_members, icmp_state)
+
+# if	defined(HAS_CONN_NEW)
+	||  CTF_MEMBER_READ(ia, ic, icmp_t_members, icmp_connp)
+# else	/* !defined(HAS_CONN_NEW) */
 	||  CTF_MEMBER_READ(ia, ic, icmp_t_members, icmp_bound_v6src)
 	||  CTF_MEMBER_READ(ia, ic, icmp_t_members, icmp_v6src)
+	||  CTF_MEMBER_READ(ia, ic, icmp_t_members, icmp_debug)
+# endif	/* defined(HAS_CONN_NEW) */
+
 	) {
 	    (void) snpf(Namech, Namechl - 1,
 		"vnode at %s; proto handle at %s; can't read icmp_t at %s",
@@ -1722,6 +1783,33 @@ read_icmp_t(va, ph, ia, ic)
 	    enter_nm(Namech);
 	    return(1);
 	}
+
+# if	defined(HAS_CONN_NEW)
+	if ((ka = (KA_T)ic->icmp_connp)
+	&& !kread(ka, (char *)&cs, sizeof(cs)))
+	{
+	    struct ip_xmit_attr_s xa;
+
+	/*
+	 * Complete the icmp_t structure from the conn_s structure.
+	 */
+	    ic->icmp_bound_v6src = cs.conn_bound_addr_v6;
+	    ic->icmp_v6src = cs.conn_saddr_v6;
+	    ic->icmp_debug.icmp_Debug = cs.conn_debug;
+	    ic->icmp_debug.icmp_broadcast = cs.conn_broadcast;
+	    ic->icmp_debug.icmp_reuseaddr = cs.conn_reuseaddr;
+	    ic->icmp_debug.icmp_useloopback = cs.conn_useloopback;
+	    ic->icmp_debug.icmp_dgram_errind = cs.conn_dgram_errind;
+	    if ((ka = (KA_T)cs.conn_ixa)
+	    &&  !kread(ka, (char *)&xa, sizeof(xa))
+	    ) {
+		ic->icmp_debug.icmp_dontroute = (xa.ixa_flags & IXAF_DONTROUTE)
+					      ? 1 : 0;
+	    }
+
+	}
+# endif	/* defined(HAS_CONN_NEW) */
+
 	return(0);
 }
 
@@ -1740,10 +1828,23 @@ read_rts_t(va, ph, ra, rt)
 {
 	char tbuf[32], tbuf1[32];	/* print_kptr() temporary buffers */
 
+# if	defined(HAS_CONN_NEW)
+	struct conn_s cs;		/* connextion structure */
+	KA_T ka;			/* kernal address */
+
+	zeromem((char *)rt, sizeof(rts_t));
+# endif	/* defined(HAS_CONN_NEW) */
+
 	(void) CTF_init(&IRU_ctfs, IRU_MOD_FORMAT, IRU_requests);
 	if (!ra
 	||  CTF_MEMBER_READ(ra, rt, rts_t_members, rts_state)
+
+# if	defined(HAS_CONN_NEW)
+	||  CTF_MEMBER_READ(ra, rt, rts_t_members, rts_connp)
+# else	/* !defined(HAS_CONN_NEW) */
 	||  CTF_MEMBER_READ(ra, rt, rts_t_members, rts_debug)
+# endif	/* defined(HAS_CONN_NEW) */
+
 	) {
 	    (void) snpf(Namech, Namechl - 1,
 		"vnode at %s; proto handle at %s; can't read rts_t at %s",
@@ -1754,6 +1855,30 @@ read_rts_t(va, ph, ra, rt)
 	    enter_nm(Namech);
 	    return(1);
 	}
+
+# if	defined(HAS_CONN_NEW)
+	if ((ka = (KA_T)rt->rts_connp)
+	&&  !kread(ka, (char *)&cs, sizeof(struct conn_s))
+	) {
+	    struct ip_xmit_attr_s xa;
+
+	/*
+	 * Fill in rts_debug from the connection structure.
+	 */
+	    rt->rts_debug.rts_Debug = cs.conn_debug;
+	    rt->rts_debug.rts_broadcast = cs.conn_broadcast;
+	    rt->rts_debug.rts_reuseaddr = cs.conn_reuseaddr;
+	    rt->rts_debug.rts_useloopback = cs.conn_useloopback;
+	    if ((ka = (KA_T)cs.conn_ixa)
+	    &&  !kread(ka, (char *)&xa, sizeof(xa))
+	    ) {
+		rt->rts_debug.rts_dontroute = (xa.ixa_flags & IXAF_DONTROUTE)
+					    ? 1 : 0;
+	    }
+	}
+
+# endif	/* defined(HAS_CONN_NEW) */
+
 	return(0);
 }
 
@@ -1770,13 +1895,18 @@ read_udp_t(ua, uc)
 	(void) CTF_init(&IRU_ctfs, IRU_MOD_FORMAT, IRU_requests);
 	if (!ua
 	||  CTF_MEMBER_READ(ua, uc, udp_t_members, udp_state)
+
+# if	defined(HAS_CONN_NEW)
+	||  CTF_MEMBER_READ(ua, uc, udp_t_members, udp_connp)
+# else	/* !defined(HAS_CONN_NEW) */
 	||  CTF_MEMBER_READ(ua, uc, udp_t_members, udp_port)
 	||  CTF_MEMBER_READ(ua, uc, udp_t_members, udp_dstport)
 	||  CTF_MEMBER_READ(ua, uc, udp_t_members, udp_v6src)
 	||  CTF_MEMBER_READ(ua, uc, udp_t_members, udp_v6dst)
 	||  CTF_MEMBER_READ(ua, uc, udp_t_members, udp_ipversion)
-	||  CTF_MEMBER_READ(ua, uc, udp_t_members, udp_connp)
 	||  CTF_MEMBER_READ(ua, uc, udp_t_members, udp_bits)
+# endif	/* defined(HAS_CONN_NEW) */
+
 	) {
 	    (void) snpf(Namech, Namechl, "can't read udp_t: %s",
 			print_kptr(ua, (char *)NULL, 0));
@@ -1840,17 +1970,53 @@ save_TCP_size(tc)
  */
 
 static void
-save_TCP_states(tc, th, tb)
+save_TCP_states(tc, fa, tb, xp)
 	tcp_t *tc;			/* pointer to TCP control structure */
-	tcph_t *th;			/* pointer to TCP header structure
-					 * (may be NULL) */
+	caddr_t *fa;			/* flags address (may be NULL):
+					 *   if HAS_CONN_NEW: conn_s *
+					 *   if !CONN_HAS_NEW: tcph_t *
+					 */
 	tcpb_t *tb;			/* pointer to TCP base structure (may
 					 * be NULL) */
+	caddr_t *xp;			/* pointer to struct ip_xmit_attr_s if
+					 * HAS_CONN_NEW (may be NULL) */
 {
 	if (!tc)
 	    return;
 
 #if	defined(HASSOOPT)
+# if	defined(HAS_CONN_NEW)
+	if (Ftcptpi & TCPTPI_FLAGS && fa) {
+	    struct conn_s *cs = (struct conn_s *)fa;
+
+	    if (cs->conn_broadcast)
+		Lf->lts.opt |= SO_BROADCAST;
+	    if (cs->conn_debug)
+		Lf->lts.opt |= SO_DEBUG;
+	    if (cs->conn_dgram_errind)
+		Lf->lts.opt |= SO_DGRAM_ERRIND;
+	    if (xp && (((ip_xmit_attr_t *)xp)->ixa_flags & IXAF_DONTROUTE))
+		Lf->lts.opt |= SO_DONTROUTE;
+	    if (cs->conn_keepalive) {
+		Lf->lts.opt |= SO_KEEPALIVE;
+		Lf->lts.kai = (unsigned int)tc->tcp_ka_interval;
+	    }
+	    if (cs->conn_linger) {
+		Lf->lts.opt |= SO_LINGER;
+		Lf->lts.ltm = (unsigned int)cs->conn_lingertime;
+	    }
+	    if (cs->conn_oobinline)
+		Lf->lts.opt |= SO_OOBINLINE;
+	    Lf->lts.pqlen = (unsigned int)tc->tcp_conn_req_cnt_q0;
+	    Lf->lts.qlen = (unsigned int)tc->tcp_conn_req_cnt_q;
+	    Lf->lts.qlim = (unsigned int)tc->tcp_conn_req_max;
+	    Lf->lts.pqlens = Lf->lts.qlens = Lf->lts.qlims
+			   = (unsigned char)1;
+	    if (cs->conn_reuseaddr)
+		Lf->lts.opt |= SO_REUSEADDR;
+	    if (cs->conn_useloopback)
+		Lf->lts.opt |= SO_USELOOPBACK;
+# else	/* !defined(HAS_CONN_NEW) */
 	if (Ftcptpi & TCPTPI_FLAGS) {
 	    if (tc->tcp_broadcast)
 		Lf->lts.opt |= SO_BROADCAST;
@@ -1876,24 +2042,27 @@ save_TCP_states(tc, th, tb)
 	    Lf->lts.pqlens = Lf->lts.qlens = Lf->lts.qlims
 			   = (unsigned char)1;
 
-# if	solaris>=80000
-#  if	defined(HAS_IPCLASSIFIER_H)
+#  if	solaris>=80000
+#   if	defined(HAS_IPCLASSIFIER_H)
 	    if (tc->tcp_reuseaddr)
-#  else	/* !defined(HAS_IPCLASSIFIER_H) */
+#   else /* !defined(HAS_IPCLASSIFIER_H) */
 	    if (tb && tb->tcpb_reuseaddr)
-#  endif	/* !defined(HAS_IPCLASSIFIER_H) */
+#   endif /* !defined(HAS_IPCLASSIFIER_H) */
 
 	    Lf->lts.opt |= SO_REUSEADDR;
-# endif	/* solaris>=80000 */
+#  endif	/* solaris>=80000 */
 
 	    if (tc->tcp_useloopback)
 		Lf->lts.opt |= SO_USELOOPBACK;
+# endif /* defined(HAS_CONN_NEW) */
 #endif	/* defined(HASSOOPT) */
 
 #if	defined(HASTCPOPT)
 # if	defined(ACK_TIMER)
-	    if (th && (th->th_flags[0] & ACK_TIMER))
+#  if	!defined(HAS_CONN_NEW)
+	    if (fa && (((tcph_t *)fa)->th_flags[0] & ACK_TIMER))
 		Lf->lts.topt |= TF_DELACK;
+#  endif /* !defined(HAS_CONN_NEW) */
 # endif	/* defined(ACK_TIMER) */
 
 # if	solaris<80000 || defined(HAS_IPCLASSIFIER_H)
