@@ -31,7 +31,7 @@
 
 
 /*
- * $Id: machine.h,v 1.44 2009/03/25 19:22:16 abe Exp $
+ * $Id: machine.h,v 1.45 2010/07/29 16:03:00 abe Exp $
  */
 
 
@@ -41,12 +41,21 @@
 
 # if	solaris>=90000
 /*
- * <sys/lgrp.h> must be #include'd for Solaris >= 9 before _KMEMUSER or
- * _KERNEL are defined to avoid a conflict with the use of lgrp_root as
- * an external symbol in <sys/lgrp.h> and a macro in <sys/lgrp_user.h>.
+ * <sys/lgrp.h> must be #include'd early on some older Solaris systems at
+ * version 9 and above before _KMEMUSER or _KERNEL are defined to avoid a
+ * conflict with the use of lgrp_root as an external symbol in <sys/lgrp.h>
+ * and a macro in <sys/lgrp_user.h>.  Some instances of those Solaris
+ * versions do not need the early #include.
+ *
+ * I have not been able to devise a Configure script test that can determine
+ * when <sys/lgrp.h> must be #include'd early.
+ *
+ * If compiling lsof on your version of Solaris reports an error in the
+ * redefinition of the lgrp_root identifier, try enabling the folowing
+ * #include.
  */
 
-#include <sys/lgrp.h>
+/* #include <sys/lgrp.h> */
 # endif	/* solaris>=90000 */
 
 
@@ -525,6 +534,13 @@ typedef struct aio_req { int dummy; } aio_req_t;
 #define	HASSNODE	1
 
 
+/*
+ * HASTASKS is defined for those dialects that have task reporting support.
+ */
+
+/* #define	HASTASKS	1 */
+
+
 /*
  * HASSOOPT, HASSOSTATE and HASTCPOPT define the availability of information
  * on socket options (SO_* symbols), socket states (SS_* symbols) and TCP
