@@ -31,7 +31,7 @@
 
 
 /*
- * $Id: lsof.h,v 1.63 2010/01/18 19:01:11 abe Exp $
+ * $Id: lsof.h,v 1.64 2010/07/29 15:59:28 abe Exp $
  */
 
 
@@ -435,6 +435,8 @@ extern int PpidColW;
 #define SZTTL		"SIZE"
 #define	SZOFFTTL	"SIZE/OFF"
 extern int SzOffColW;
+#define	TIDTTL		"TID"
+extern	int TidColW;
 #define TYPETTL		"TYPE"
 extern int TypeColW;
 #define	USERTTL		"USER"
@@ -465,8 +467,9 @@ extern int ZoneColW;
 #define	SELUNX		0x0800		/* select UNIX socket (-U) */
 #define	SELZONE		0x1000		/* select zone (-z) */
 #define	SELEXCLF	0x2000		/* file selection excluded */
-#define	SELALL		(SELCMD|SELCNTX|SELFD|SELNA|SELNET|SELNM|SELNFS|SELPID|SELUID|SELUNX|SELZONE)
-#define	SELPROC		(SELCMD|SELCNTX|SELPGID|SELPID|SELUID|SELZONE)
+#define	SELTASK		0x4000		/* select tasks (-K) */
+#define	SELALL		(SELCMD|SELCNTX|SELFD|SELNA|SELNET|SELNM|SELNFS|SELPID|SELUID|SELUNX|SELZONE|SELTASK)
+#define	SELPROC		(SELCMD|SELCNTX|SELPGID|SELPID|SELUID|SELZONE|SELTASK)
 					/* process selecters */
 #define	SELFILE		(SELFD|SELNFS|SELNLINK|SELNM)	/* file selecters */
 #define	SELNW		(SELNA|SELNET|SELUNX)		/* network selecters */
@@ -613,6 +616,7 @@ extern int Fsize;
 extern int Fsv;
 extern int FsvByf;
 extern int FsvFlagX;
+extern int Ftask;
 extern int Ftcptpi;
 extern int Fterse;
 extern int Funix;
@@ -817,6 +821,11 @@ struct lproc {
 				 	 *	  1 = wholly selected
 				 	 *	  2 = partially selected */
 	int pid;			/* process ID */
+
+# if	defined(HASTASKS)
+	int tid;			/* task ID */
+# endif	/* HASTASKS */
+
 	int pgid;			/* process group ID */
 	int ppid;			/* parent process ID */
 	uid_t uid;			/* user ID */
@@ -924,6 +933,7 @@ extern char *SzOffFmt_0t;
 extern char *SzOffFmt_d;
 extern char *SzOffFmt_dv;
 extern char *SzOffFmt_x;
+extern int TaskPrtFl;
 extern int TcpStAlloc;
 extern unsigned char *TcpStI;
 extern int TcpStIn;
