@@ -34,7 +34,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: main.c,v 1.53 2008/10/21 16:21:41 abe Exp $";
+static char *rcsid = "$Id: main.c,v 1.54 2010/07/29 15:59:28 abe Exp $";
 #endif
 
 
@@ -154,7 +154,7 @@ main(argc, argv)
  * Create option mask.
  */
 	(void) snpf(options, sizeof(options),
-	    "?a%sbc:D:d:%sf:F:g:hi:%slL:%sMnNo:Op:Pr:%ss:S:tT:u:UvVwx:%s%s%s",
+	    "?a%sbc:D:d:%sf:F:g:hi:%s%slL:%sMnNo:Op:Pr:%ss:S:tT:u:UvVwx:%s%s%s",
 
 #if	defined(HAS_AFS) && defined(HASAOPT)
 	    "A:",
@@ -174,6 +174,12 @@ main(argc, argv)
 	    "",
 #endif	/* defined(HASKOPT) */
 
+#if	defined(HASTASKS)
+	    "K",
+#else	/* !defined(HASTASKS) */
+	    "",
+#endif	/* defined(HASTASKS) */
+
 #if	defined(HASMOPT) || defined(HASMNTSUP)
 	    "m:",
 #else	/* !defined(HASMOPT) && !defined(HASMNTSUP) */
@@ -424,11 +430,6 @@ main(argc, argv)
 			||  FieldSel[i].id == LSOF_FID_NI)
 			    continue;
 #endif	/* !defined(HASFSTRUCT) */
-
-#if	!defined(HASZONES)
-			if (FieldSel[i].id == LSOF_FID_ZONE)
-			    continue;
-#endif	/* !defined(HASZONES) */
  
 #if	defined(HASSELINUX)
 			if ((FieldSel[i].id == LSOF_FID_CNTX) && !CntxStatus)
@@ -440,6 +441,17 @@ main(argc, argv)
 
 			if (FieldSel[i].id == LSOF_FID_RDEV)
 			    continue;	/* for compatibility */
+
+#if	!defined(HASTASKS)
+			if (FieldSel[i].id == LSOF_FID_TID)
+			    continue;
+#endif	/* !defined(HASTASKS) */
+
+#if	!defined(HASZONES)
+			if (FieldSel[i].id == LSOF_FID_ZONE)
+			    continue;
+#endif	/* !defined(HASZONES) */
+
 			FieldSel[i].st = 1;
 			if (FieldSel[i].opt && FieldSel[i].ov)
 			    *(FieldSel[i].opt) |= FieldSel[i].ov;
@@ -473,6 +485,11 @@ main(argc, argv)
 			    continue;
 #endif	/* !defined(HASFSTRUCT) */
 
+#if	!defined(HASTASKS)
+			if (FieldSel[i].id == LSOF_FID_TID)
+			    continue;
+#endif	/* !defined(HASTASKS) */
+
 			if (FieldSel[i].id == *GOv) {
 			    FieldSel[i].st = 1;
 			    if (FieldSel[i].opt && FieldSel[i].ov)
@@ -538,6 +555,13 @@ main(argc, argv)
 		break;
 #endif	/* defined(HASKOPT) */
 
+#if	defined(HASTASKS)
+		case 'K':
+		    Ftask = 1;
+		    Selflags |= SELTASK;
+		    break;
+#endif	/* defined(HASTASKS) */
+
 	    case 'l':
 		Futol = 0;
 		break;
@@ -1451,6 +1475,18 @@ main(argc, argv)
 		    Pn, Spid[i].i);
 	}
 
+#if	defined(HASTASKS)
+	if (Ftask && Ftask < 2) {
+
+	/*
+	 * Report no tasks located.
+	 */
+	    rv = 1;
+	    if (Fverbose)
+		(void) printf("%s: no tasks located\n", Pn);
+	}
+#endif	/* defined(HASTASKS) */
+
 #if	defined(HASZONES)
 	if (ZoneArg) {
 
